@@ -18,26 +18,26 @@
 
 #define G_LOG_DOMAIN "ide-editor-perspective"
 
+#include <egg-signal-group.h>
 #include <glib/gi18n.h>
 
-#include "egg-signal-group.h"
-
-#include "ide-buffer.h"
-#include "ide-buffer-manager.h"
 #include "ide-context.h"
 #include "ide-debug.h"
-#include "ide-editor-perspective.h"
-#include "ide-editor-view.h"
-#include "ide-gtk.h"
-#include "ide-layout-grid.h"
-#include "ide-workbench-header-bar.h"
+
+#include "buffers/ide-buffer-manager.h"
+#include "buffers/ide-buffer.h"
+#include "editor/ide-editor-perspective.h"
+#include "editor/ide-editor-view.h"
+#include "util/ide-gtk.h"
+#include "workbench/ide-layout-grid.h"
+#include "workbench/ide-workbench.h"
+#include "workbench/ide-workbench-header-bar.h"
 
 struct _IdeEditorPerspective
 {
   IdeLayout              parent_instance;
 
   IdeLayoutGrid         *grid;
-  IdeWorkbenchHeaderBar *titlebar;
   GSimpleActionGroup    *actions;
 
   EggSignalGroup        *buffer_manager_signals;
@@ -49,9 +49,12 @@ typedef struct
   IdeSourceLocation    *location;
 } FocusLocation;
 
-static void ide_perspective_iface_init (IdePerspectiveInterface *iface);
-
-static void ide_editor_perspective_add (GtkContainer *container, GtkWidget *widget);
+static void ide_perspective_iface_init                 (IdePerspectiveInterface *iface);
+static void ide_editor_perspective_add                 (GtkContainer            *container,
+                                                        GtkWidget               *widget);
+static void ide_editor_perspective_focus_location_full (IdeEditorPerspective    *self,
+                                                        IdeSourceLocation       *location,
+                                                        gboolean                 open_if_not_found);
 
 G_DEFINE_TYPE_EXTENDED (IdeEditorPerspective, ide_editor_perspective, IDE_TYPE_LAYOUT, 0,
                         G_IMPLEMENT_INTERFACE (IDE_TYPE_PERSPECTIVE, ide_perspective_iface_init))
@@ -145,7 +148,7 @@ ide_editor_perspective_context_set (GtkWidget  *widget,
 static void
 ide_editor_perspective_load_buffer (IdeEditorPerspective *self,
                                     IdeBuffer            *buffer,
-                                    gboolean              reloading,
+                                    gboolean              create_new_view,
                                     IdeBufferManager     *buffer_manager)
 {
   IdeEditorView *view;
@@ -160,7 +163,7 @@ ide_editor_perspective_load_buffer (IdeEditorPerspective *self,
    * We only want to create a new view when the buffer is originally
    * created, not when it's reloaded.
    */
-  if (reloading)
+  if (!create_new_view)
     {
       ide_buffer_manager_set_focus_buffer (buffer_manager, buffer);
       return;
@@ -267,18 +270,6 @@ ide_editor_perspective_notify_focus_buffer (IdeEditorPerspective *self,
                                 &buffer);
 }
 
-static void
-global_search_activate (GSimpleAction *action,
-                        GVariant      *param,
-                        gpointer       user_data)
-{
-  IdeEditorPerspective *self = user_data;
-
-  g_assert (IDE_IS_EDITOR_PERSPECTIVE (self));
-
-  ide_workbench_header_bar_focus_search (self->titlebar);
-}
-
 static void
 new_file_activate (GSimpleAction *action,
                    GVariant      *param,
@@ -372,7 +363,6 @@ ide_editor_perspective_class_init (IdeEditorPerspectiveClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/builder/ui/ide-editor-perspective.ui");
   gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, actions);
   gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, grid);
-  gtk_widget_class_bind_template_child (widget_class, IdeEditorPerspective, titlebar);
 
   signals[VIEW_ADDED] =
     g_signal_new ("view-added",
@@ -398,12 +388,19 @@ ide_editor_perspective_class_init (IdeEditorPerspectiveClass *klass)
 static void
 ide_editor_perspective_init (IdeEditorPerspective *self)
 {
-  GActionGroup *actions;
+  static const gchar *proxy_actions[] = {
+    "bottom-visible",
+    "left-visible",
+    "right-visible",
+    NULL
+  };
   static const GActionEntry entries[] = {
     { "new-file", new_file_activate },
-    { "global-search", global_search_activate },
   };
 
+  GActionGroup *actions;
+  guint i;
+
   self->buffer_manager_signals = egg_signal_group_new (IDE_TYPE_BUFFER_MANAGER);
 
   egg_signal_group_connect_object (self->buffer_manager_signals,
@@ -430,7 +427,14 @@ ide_editor_perspective_init (IdeEditorPerspective *self)
                                    G_N_ELEMENTS (entries), self);
 
   actions = gtk_widget_get_action_group (GTK_WIDGET (self), "dockbin");
-  gtk_widget_insert_action_group (GTK_WIDGET (self->titlebar), "dockbin", actions);
+
+  for (i = 0; proxy_actions[i]; i++)
+    {
+      GAction *action;
+
+      action = g_action_map_lookup_action (G_ACTION_MAP (actions), proxy_actions[i]);
+      g_action_map_add_action (G_ACTION_MAP (self->actions), action);
+    }
 
   ide_editor_perspective_restore_panel_state (self);
 
@@ -444,16 +448,10 @@ ide_editor_perspective_get_title (IdePerspective *perspective)
   return g_strdup (_("Editor"));
 }
 
-static GtkWidget *
-ide_editor_perspective_get_titlebar (IdePerspective *perspective)
-{
-  return GTK_WIDGET (IDE_EDITOR_PERSPECTIVE (perspective)->titlebar);
-}
-
 static gchar *
 ide_editor_perspective_get_icon_name (IdePerspective *perspective)
 {
-  return g_strdup ("text-editor-symbolic");
+  return g_strdup ("builder-editor-symbolic");
 }
 
 static gchar *
@@ -496,6 +494,12 @@ ide_editor_perspective_agree_to_shutdown (IdePerspective *perspective)
   return TRUE;
 }
 
+static gchar *
+ide_editor_perspective_get_accelerator (IdePerspective *perspective)
+{
+  return g_strdup ("<alt>1");
+}
+
 static void
 ide_perspective_iface_init (IdePerspectiveInterface *iface)
 {
@@ -504,8 +508,8 @@ ide_perspective_iface_init (IdePerspectiveInterface *iface)
   iface->get_icon_name = ide_editor_perspective_get_icon_name;
   iface->get_id = ide_editor_perspective_get_id;
   iface->get_title = ide_editor_perspective_get_title;
-  iface->get_titlebar = ide_editor_perspective_get_titlebar;
   iface->views_foreach = ide_editor_perspective_views_foreach;
+  iface->get_accelerator = ide_editor_perspective_get_accelerator;
 }
 
 static void
@@ -557,7 +561,7 @@ ide_editor_perspective_focus_location_cb (GObject      *object,
     }
 
   /* try again now that we have loaded */
-  ide_editor_perspective_focus_location (state->self, state->location);
+  ide_editor_perspective_focus_location_full (state->self, state->location, FALSE);
 
 cleanup:
   g_object_unref (state->self);
@@ -565,9 +569,10 @@ cleanup:
   g_slice_free (FocusLocation, state);
 }
 
-void
-ide_editor_perspective_focus_location (IdeEditorPerspective *self,
-                                       IdeSourceLocation    *location)
+static void
+ide_editor_perspective_focus_location_full (IdeEditorPerspective *self,
+                                            IdeSourceLocation    *location,
+                                            gboolean              open_if_not_found)
 {
   struct {
     IdeFile *file;
@@ -585,6 +590,9 @@ ide_editor_perspective_focus_location (IdeEditorPerspective *self,
                                  ide_editor_perspective_find_source_location,
                                  &lookup);
 
+  if (!open_if_not_found && lookup.view == NULL)
+    return;
+
   if (lookup.view == NULL)
     {
       FocusLocation *state;
@@ -603,6 +611,7 @@ ide_editor_perspective_focus_location (IdeEditorPerspective *self,
       ide_buffer_manager_load_file_async (bufmgr,
                                           lookup.file,
                                           FALSE,
+                                          IDE_WORKBENCH_OPEN_FLAGS_NONE,
                                           NULL,
                                           NULL,
                                           ide_editor_perspective_focus_location_cb,
@@ -615,3 +624,10 @@ ide_editor_perspective_focus_location (IdeEditorPerspective *self,
   ide_layout_view_navigate_to (IDE_LAYOUT_VIEW (lookup.view), location);
   gtk_widget_grab_focus (GTK_WIDGET (lookup.view));
 }
+
+void
+ide_editor_perspective_focus_location (IdeEditorPerspective *self,
+                                       IdeSourceLocation    *location)
+{
+  ide_editor_perspective_focus_location_full (self, location, TRUE);
+}
