@@ -21,34 +21,37 @@
 #include <glib/gi18n.h>
 #include <libpeas/peas.h>
 
-#include "ide-async-helper.h"
-#include "ide-back-forward-list.h"
-#include "ide-back-forward-list-private.h"
-#include "ide-buffer-manager.h"
-#include "ide-buffer.h"
-#include "ide-build-system.h"
-#include "ide-configuration-manager.h"
 #include "ide-context.h"
 #include "ide-debug.h"
-#include "ide-device-manager.h"
 #include "ide-global.h"
 #include "ide-internal.h"
-#include "ide-project.h"
-#include "ide-project-item.h"
-#include "ide-project-files.h"
-#include "ide-runtime-manager.h"
-#include "ide-script-manager.h"
-#include "ide-search-engine.h"
-#include "ide-search-provider.h"
 #include "ide-service.h"
-#include "ide-settings.h"
-#include "ide-source-snippets-manager.h"
-#include "ide-unsaved-file.h"
-#include "ide-unsaved-files.h"
-#include "ide-vcs.h"
-#include "ide-recent-projects.h"
 
+#include "buffers/ide-buffer-manager.h"
+#include "buffers/ide-buffer.h"
+#include "buffers/ide-unsaved-file.h"
+#include "buffers/ide-unsaved-files.h"
+#include "buildsystem/ide-build-manager.h"
+#include "buildsystem/ide-build-system.h"
+#include "buildsystem/ide-configuration-manager.h"
+#include "devices/ide-device-manager.h"
 #include "doap/ide-doap.h"
+#include "history/ide-back-forward-list-private.h"
+#include "history/ide-back-forward-list.h"
+#include "projects/ide-project-files.h"
+#include "projects/ide-project-item.h"
+#include "projects/ide-project.h"
+#include "projects/ide-recent-projects.h"
+#include "runner/ide-run-manager.h"
+#include "runtimes/ide-runtime-manager.h"
+#include "scripting/ide-script-manager.h"
+#include "search/ide-search-engine.h"
+#include "search/ide-search-provider.h"
+#include "snippets/ide-source-snippets-manager.h"
+#include "util/ide-async-helper.h"
+#include "util/ide-settings.h"
+#include "vcs/ide-vcs.h"
+#include "workbench/ide-workbench.h"
 
 #define RESTORE_FILES_MAX_FILES 20
 
@@ -58,11 +61,13 @@ struct _IdeContext
 
   IdeBackForwardList       *back_forward_list;
   IdeBufferManager         *buffer_manager;
+  IdeBuildManager          *build_manager;
   IdeBuildSystem           *build_system;
   IdeConfigurationManager  *configuration_manager;
   IdeDeviceManager         *device_manager;
   IdeDoap                  *doap;
   GtkRecentManager         *recent_manager;
+  IdeRunManager            *run_manager;
   IdeRuntimeManager        *runtime_manager;
   IdeScriptManager         *script_manager;
   IdeSearchEngine          *search_engine;
@@ -139,7 +144,7 @@ ide_context_get_recent_manager (IdeContext *self)
  * Retrieves the global back forward list for the #IdeContext.
  *
  * Consumers of this should branch the #IdeBackForwardList and merge them
- * when there document stack is closed.
+ * when their document stack is closed.
  *
  * See ide_back_forward_list_branch() and ide_back_forward_list_merge() for
  * more information.
@@ -171,6 +176,19 @@ ide_context_get_buffer_manager (IdeContext *self)
   return self->buffer_manager;
 }
 
+/**
+ * ide_context_get_build_manager:
+ *
+ * Returns: (transfer none): An #IdeBuildManager.
+ */
+IdeBuildManager *
+ide_context_get_build_manager (IdeContext *self)
+{
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
+
+  return self->build_manager;
+}
+
 /**
  * ide_context_get_build_system:
  *
@@ -810,6 +828,10 @@ ide_context_init (IdeContext *self)
                                        "context", self,
                                        NULL);
 
+  self->build_manager = g_object_new (IDE_TYPE_BUILD_MANAGER,
+                                      "context", self,
+                                      NULL);
+
   self->device_manager = g_object_new (IDE_TYPE_DEVICE_MANAGER,
                                        "context", self,
                                        NULL);
@@ -822,6 +844,10 @@ ide_context_init (IdeContext *self)
                                 "context", self,
                                 NULL);
 
+  self->run_manager = g_object_new (IDE_TYPE_RUN_MANAGER,
+                                    "context", self,
+                                    NULL);
+
   self->runtime_manager = g_object_new (IDE_TYPE_RUNTIME_MANAGER,
                                         "context", self,
                                         NULL);
@@ -1898,6 +1924,7 @@ restore_in_idle (gpointer user_data)
   IdeContext *self;
   GPtrArray *ar;
   GFile *file;
+  IdeWorkbenchOpenFlags flags;
 
   g_assert (G_IS_TASK (task));
 
@@ -1919,9 +1946,12 @@ restore_in_idle (gpointer user_data)
   ifile = ide_project_get_project_file (self->project, file);
   g_ptr_array_remove_index (ar, ar->len - 1);
 
+  flags = IDE_WORKBENCH_OPEN_FLAGS_NONE;
+
   ide_buffer_manager_load_file_async (self->buffer_manager,
                                       ifile,
                                       FALSE,
+                                      flags,
                                       NULL,
                                       g_task_get_cancellable (task),
                                       ide_context_restore__load_file_cb,
@@ -2147,3 +2177,21 @@ ide_context_warning (IdeContext  *self,
   g_logv ("Ide", G_LOG_LEVEL_WARNING, format, args);
   va_end (args);
 }
+
+/**
+ * ide_context_get_run_manager:
+ *
+ * Gets the #IdeRunManager for the context. This manager object simplifies
+ * the process of running an #IdeBuildTarget from the build system. Primarily,
+ * it enforces that only a single target may be run at a time, since that is
+ * what the UI will expect.
+ *
+ * Returns: (transfer none): An #IdeRunManager.
+ */
+IdeRunManager *
+ide_context_get_run_manager (IdeContext *self)
+{
+  g_return_val_if_fail (IDE_IS_CONTEXT (self), NULL);
+
+  return self->run_manager;
+}
