@@ -36,22 +36,18 @@ typedef struct _IdeBufferChangeMonitor         IdeBufferChangeMonitor;
 typedef struct _IdeBufferManager               IdeBufferManager;
 
 typedef struct _IdeBuilder                     IdeBuilder;
-
+typedef struct _IdeBuildManager                IdeBuildManager;
 typedef struct _IdeBuildResult                 IdeBuildResult;
-
 typedef struct _IdeBuildSystem                 IdeBuildSystem;
+typedef struct _IdeBuildTarget                 IdeBuildTarget;
+
+typedef struct _IdeConfiguration               IdeConfiguration;
+typedef struct _IdeConfigurationManager        IdeConfigurationManager;
 
 typedef struct _IdeContext                     IdeContext;
 
-typedef struct _IdeDebugger                    IdeDebugger;
-typedef struct _IdeDebuggerInterface           IdeDebuggerInterface;
-
-typedef struct _IdeDeployer                    IdeDeployer;
-
 typedef struct _IdeDevice                      IdeDevice;
-
 typedef struct _IdeDeviceManager               IdeDeviceManager;
-
 typedef struct _IdeDeviceProvider              IdeDeviceProvider;
 
 typedef struct _IdeDiagnostic                  IdeDiagnostic;
@@ -62,11 +58,8 @@ typedef struct _IdeDiagnostician               IdeDiagnostician;
 
 typedef struct _IdeDiagnosticProvider          IdeDiagnosticProvider;
 
-typedef struct _IdeExecuter                    IdeExecuter;
-typedef struct _IdeExecuterInterface           IdeExecuterInterface;
-
-typedef struct _IdeExecutable                  IdeExecutable;
-typedef struct _IdeExecutableInterface         IdeExecutableInterface;
+typedef struct _IdeEnvironment                 IdeEnvironment;
+typedef struct _IdeEnvironmentVariable         IdeEnvironmentVariable;
 
 typedef struct _IdeFile                        IdeFile;
 
@@ -78,13 +71,8 @@ typedef struct _IdeHighlighter                 IdeHighlighter;
 
 typedef struct _IdeIndenter                    IdeIndenter;
 
-typedef struct _IdeLanguage                    IdeLanguage;
-
 typedef struct _IdeObject                      IdeObject;
 
-typedef struct _IdeProcess                     IdeProcess;
-typedef struct _IdeProcessInterface            IdeProcessInterface;
-
 typedef struct _IdeProgress                    IdeProgress;
 
 typedef struct _IdeProject                     IdeProject;
@@ -95,8 +83,10 @@ typedef struct _IdeProjectFile                 IdeProjectFile;
 
 typedef struct _IdeProjectFiles                IdeProjectFiles;
 
-typedef struct _IdeRefactory                   IdeRefactory;
-typedef struct _IdeRefactoryInterface          IdeRefactoryInterface;
+typedef struct _IdeRunManager                  IdeRunManager;
+typedef struct _IdeRuntime                     IdeRuntime;
+typedef struct _IdeRuntimeManager              IdeRuntimeManager;
+typedef struct _IdeRuntimeProvider             IdeRuntimeProvider;
 
 typedef struct _IdeScript                      IdeScript;
 
@@ -133,15 +123,6 @@ typedef struct _IdeSymbol                      IdeSymbol;
 typedef struct _IdeSymbolResolver              IdeSymbolResolver;
 typedef struct _IdeSymbolResolverInterface     IdeSymbolResolverInterface;
 
-typedef struct _IdeTarget                      IdeTarget;
-typedef struct _IdeTargetInterface             IdeTargetInterface;
-
-typedef struct _IdeTestCase                    IdeTestCase;
-typedef struct _IdeTestCaseInterface           IdeTestCaseInterface;
-
-typedef struct _IdeTestSuite                   IdeTestSuite;
-typedef struct _IdeTestSuiteInterface          IdeTestSuiteInterface;
-
 typedef struct _IdeUnsavedFiles                IdeUnsavedFiles;
 
 typedef struct _IdeUnsavedFile                 IdeUnsavedFile;
