@@ -16,14 +16,20 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "ide-preferences-builtin"
+
 #include <glib/gi18n.h>
 #include <gtksourceview/gtksource.h>
 #include <libpeas/peas.h>
 
 #include "ide-macros.h"
-#include "ide-preferences-builtin.h"
-#include "ide-preferences-entry.h"
-#include "ide-preferences-language-row.h"
+
+#include "application/ide-application-private.h"
+#include "preferences/ide-preferences-builtin.h"
+#include "preferences/ide-preferences-entry.h"
+#include "preferences/ide-preferences-language-row.h"
+#include "preferences/ide-preferences-spin-button.h"
+#include "vcs/ide-vcs-config.h"
 
 static void
 ide_preferences_builtin_register_plugins (IdePreferences *preferences)
@@ -72,12 +78,14 @@ ide_preferences_builtin_register_appearance (IdePreferences *preferences)
 {
   GtkSourceStyleSchemeManager *manager;
   const gchar * const *scheme_ids;
+  GtkWidget *bin;
   gint i;
+  gint dark_mode;
 
   ide_preferences_add_page (preferences, "appearance", _("Appearance"), 0);
 
   ide_preferences_add_list_group (preferences, "appearance", "basic", _("Themes"), 0);
-  ide_preferences_add_switch (preferences, "appearance", "basic", "org.gnome.builder", "night-mode", NULL, NULL, _("Dark Theme"), _("Whether Builder should use a dark theme"), _("dark theme"), 0);
+  dark_mode = ide_preferences_add_switch (preferences, "appearance", "basic", "org.gnome.builder", "night-mode", NULL, NULL, _("Dark Theme"), _("Whether Builder should use a dark theme"), _("dark theme"), 0);
   ide_preferences_add_switch (preferences, "appearance", "basic", "org.gnome.builder.editor", "show-grid-lines", NULL, NULL, _("Grid Pattern"), _("Display a grid pattern underneath source code"), NULL, 0);
 
   ide_preferences_add_list_group (preferences, "appearance", "schemes", NULL, 100);
@@ -102,6 +110,12 @@ ide_preferences_builtin_register_appearance (IdePreferences *preferences)
   ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.editor", "font-name", _("Editor"), C_("Keywords", "editor font monospace"), 0);
   /* XXX: This belongs in terminal addin */
   ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.terminal", "font-name", _("Terminal"), C_("Keywords", "terminal font monospace"), 0);
+
+  if (ide_application_get_disable_theme_tracking (IDE_APPLICATION_DEFAULT))
+    {
+      bin = ide_preferences_get_widget (preferences, dark_mode);
+      gtk_widget_set_sensitive (bin, FALSE);
+    }
 }
 
 static void
@@ -237,13 +251,61 @@ ide_preferences_builtin_register_languages (IdePreferences *preferences)
   ide_preferences_add_radio (preferences, "languages.id", "indentation", "org.gnome.builder.editor.language", "auto-indent", "/org/gnome/builder/editor/language/{id}/", NULL, _("Automatically indent"), _("Indent source code as you type"), NULL, 30);
 }
 
+static gboolean
+workers_output (GtkSpinButton *button)
+{
+  GtkAdjustment *adj = gtk_spin_button_get_adjustment (button);
+
+  if (gtk_adjustment_get_value (adj) == -1)
+    {
+      gtk_entry_set_text (GTK_ENTRY (button), _("Default"));
+      return TRUE;
+    }
+  else if (gtk_adjustment_get_value (adj) == 0)
+    {
+      gtk_entry_set_text (GTK_ENTRY (button), _("Number of CPU"));
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gint
+workers_input (GtkSpinButton *button,
+               gdouble       *new_value)
+{
+  const gchar *text = gtk_entry_get_text (GTK_ENTRY (button));
+
+  if (g_strcmp0 (text, _("Default")) == 0)
+    {
+      *new_value = -1;
+      return TRUE;
+    }
+  else if (g_strcmp0 (text, _("Number of CPU")) == 0)
+    {
+      *new_value = 0;
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 ide_preferences_builtin_register_build (IdePreferences *preferences)
 {
+  GtkWidget *widget, *bin;
+  guint id;
+
   ide_preferences_add_page (preferences, "build", _("Build"), 500);
 
   ide_preferences_add_list_group (preferences, "build", "basic", _("General"), 0);
-  ide_preferences_add_spin_button (preferences, "build", "basic", "org.gnome.builder.build", "parallel", "/org/gnome/builder/build/", _("Build Workers"), _("Number of parallel build workers"), NULL, 0);
+  id = ide_preferences_add_spin_button (preferences, "build", "basic", "org.gnome.builder.build", "parallel", "/org/gnome/builder/build/", _("Build Workers"), _("Number of parallel build workers"), NULL, 0);
+
+  bin = ide_preferences_get_widget (preferences, id);
+  widget = ide_preferences_spin_button_get_spin_button (IDE_PREFERENCES_SPIN_BUTTON (bin));
+  gtk_entry_set_width_chars (GTK_ENTRY (widget), 20);
+  g_signal_connect (widget, "input", G_CALLBACK (workers_input), NULL);
+  g_signal_connect (widget, "output", G_CALLBACK (workers_output), NULL);
 }
 
 static void
@@ -252,12 +314,139 @@ ide_preferences_builtin_register_projects (IdePreferences *preferences)
   ide_preferences_add_page (preferences, "projects", _("Projects"), 450);
 
   ide_preferences_add_list_group (preferences, "projects", "directory", _("Workspace"), 0);
-  ide_preferences_add_file_chooser (preferences, "projects", "directory", "org.gnome.builder", "projects-directory", NULL, _("Projects Directory"), _("A place for all your projects"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, NULL, 0);
+  ide_preferences_add_file_chooser (preferences, "projects", "directory", "org.gnome.builder", "projects-directory", NULL, _("Projects directory"), _("A place for all your projects"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, NULL, 0);
+  ide_preferences_add_switch (preferences, "projects", "directory", "org.gnome.builder", "restore-previous-files", NULL, NULL, _("Restore previously opened files"), _("Open previously opened files when loading a project"), NULL, 10);
 
   ide_preferences_add_list_group (preferences, "projects", "discovery", _("Project Discovery"), 0);
   ide_preferences_add_switch (preferences, "projects", "discovery", "org.gnome.builder", "enable-project-miners", NULL, NULL, _("Discover projects on my computer"), _("Scan your computer for existing projects"), NULL, 0);
 }
 
+static void
+author_changed_cb (IdePreferencesEntry *entry,
+                   const gchar         *text,
+                   IdeVcsConfig        *conf)
+{
+  GValue value = G_VALUE_INIT;
+
+  g_assert (IDE_IS_PREFERENCES_ENTRY (entry));
+  g_assert (text != NULL);
+  g_assert (IDE_IS_VCS_CONFIG (conf));
+
+  g_value_init (&value, G_TYPE_STRING);
+  g_value_set_string (&value, text);
+
+  ide_vcs_config_set_config (conf, IDE_VCS_CONFIG_FULL_NAME, &value);
+
+  g_value_unset (&value);
+}
+
+static void
+email_changed_cb (IdePreferencesEntry *entry,
+                  const gchar         *text,
+                  IdeVcsConfig        *conf)
+{
+  GValue value = G_VALUE_INIT;
+
+  g_assert (IDE_IS_PREFERENCES_ENTRY (entry));
+  g_assert (text != NULL);
+  g_assert (IDE_IS_VCS_CONFIG (conf));
+
+  g_value_init (&value, G_TYPE_STRING);
+  g_value_set_string (&value, text);
+
+  ide_vcs_config_set_config (conf, IDE_VCS_CONFIG_EMAIL, &value);
+
+  g_value_unset (&value);
+}
+
+static void
+vcs_configs_foreach_cb (PeasExtensionSet *set,
+                        PeasPluginInfo   *plugin_info,
+                        PeasExtension    *exten,
+                        gpointer          user_data)
+{
+  IdePreferences *preferences = user_data;
+  IdeVcsConfig *conf = (IdeVcsConfig *)exten;
+  GValue value = G_VALUE_INIT;
+  GtkWidget *fullname;
+  GtkWidget *email;
+  GtkSizeGroup *size_group;
+  g_autofree gchar *key = NULL;
+  g_autofree gchar *author_name = NULL;
+  g_autofree gchar *author_email = NULL;
+  const gchar *name;
+  const gchar *id;
+
+  g_assert (PEAS_IS_EXTENSION_SET (set));
+  g_assert (IDE_IS_PREFERENCES (preferences));
+  g_assert (IDE_IS_VCS_CONFIG (conf));
+
+  name = peas_plugin_info_get_name (plugin_info);
+  id = peas_plugin_info_get_module_name (plugin_info);
+  key = g_strdup_printf ("%s-config", id);
+
+  g_object_set_data_full (G_OBJECT (preferences), key, g_object_ref (conf), g_object_unref);
+
+  g_value_init (&value, G_TYPE_STRING);
+
+  ide_vcs_config_get_config (conf, IDE_VCS_CONFIG_FULL_NAME, &value);
+  author_name = g_strdup (g_value_get_string (&value));
+
+  g_value_reset (&value);
+
+  ide_vcs_config_get_config (conf, IDE_VCS_CONFIG_EMAIL, &value);
+  author_email = g_strdup (g_value_get_string (&value));
+
+  g_value_unset (&value);
+
+  fullname = g_object_new (IDE_TYPE_PREFERENCES_ENTRY,
+                           "text", ide_str_empty0 (author_name) ? "" : author_name,
+                           "title", "Author",
+                           "visible", TRUE,
+                           NULL);
+
+  g_signal_connect_object (fullname,
+                           "changed",
+                           G_CALLBACK (author_changed_cb),
+                           conf,
+                           0);
+
+  email = g_object_new (IDE_TYPE_PREFERENCES_ENTRY,
+                        "text", ide_str_empty0 (author_email) ? "" : author_email,
+                        "title", "Email",
+                        "visible", TRUE,
+                        NULL);
+
+  g_signal_connect_object (email,
+                           "changed",
+                           G_CALLBACK (email_changed_cb),
+                           conf,
+                           0);
+
+  ide_preferences_add_list_group (preferences, "vcs", id, name, 0);
+  ide_preferences_add_custom (preferences, "vcs", id, fullname, NULL, 0);
+  ide_preferences_add_custom (preferences, "vcs", id, email, NULL, 0);
+
+  size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+  gtk_size_group_add_widget (size_group, ide_preferences_entry_get_title_widget (IDE_PREFERENCES_ENTRY (fullname)));
+  gtk_size_group_add_widget (size_group, ide_preferences_entry_get_title_widget (IDE_PREFERENCES_ENTRY (email)));
+  g_clear_object (&size_group);
+}
+
+static void
+ide_preferences_builtin_register_vcs (IdePreferences *preferences)
+{
+  PeasEngine *engine;
+  PeasExtensionSet *extensions;
+
+  ide_preferences_add_page (preferences, "vcs", _("Version Control"), 600);
+
+  engine = peas_engine_get_default ();
+  extensions = peas_extension_set_new (engine, IDE_TYPE_VCS_CONFIG, NULL);
+  peas_extension_set_foreach (extensions, vcs_configs_foreach_cb, preferences);
+  g_clear_object (&extensions);
+}
+
 void
 _ide_preferences_builtin_register (IdePreferences *preferences)
 {
@@ -270,4 +459,5 @@ _ide_preferences_builtin_register (IdePreferences *preferences)
   ide_preferences_builtin_register_plugins (preferences);
   ide_preferences_builtin_register_build (preferences);
   ide_preferences_builtin_register_projects (preferences);
+  ide_preferences_builtin_register_vcs (preferences);
 }
