@@ -18,32 +18,19 @@
 
 #define G_LOG_DOMAIN "ide-autotools-build-system"
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "config.h"
 
+#include <egg-counter.h>
+#include <egg-task-cache.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gtksourceview/gtksource.h>
-
-#include "egg-counter.h"
-#include "egg-task-cache.h"
+#include <ide.h>
+#include <ide-internal.h>
 
 #include "ide-autotools-build-system.h"
 #include "ide-autotools-builder.h"
-#include "ide-buffer-manager.h"
-#include "ide-configuration.h"
-#include "ide-configuration-manager.h"
-#include "ide-context.h"
-#include "ide-debug.h"
-#include "ide-device.h"
-#include "ide-device-manager.h"
-#include "ide-file.h"
-#include "ide-internal.h"
 #include "ide-makecache.h"
-#include "ide-runtime.h"
-#include "ide-runtime-manager.h"
-#include "ide-tags-builder.h"
 
 #define MAKECACHE_KEY "makecache"
 #define DEFAULT_MAKECACHE_TTL 0
@@ -528,18 +515,40 @@ ide_autotools_build_system__buffer_saved_cb (IdeAutotoolsBuildSystem *self,
     egg_task_cache_evict (self->task_cache, MAKECACHE_KEY);
 }
 
+static void
+ide_autotools_build_system__config_changed_cb (IdeAutotoolsBuildSystem *self,
+                                               GParamSpec              *pspec,
+                                               IdeConfigurationManager *config_manager)
+{
+  g_assert (IDE_IS_AUTOTOOLS_BUILD_SYSTEM (self));
+  g_assert (IDE_IS_CONFIGURATION_MANAGER (config_manager));
+
+  egg_task_cache_evict (self->task_cache, MAKECACHE_KEY);
+}
+
 static void
 ide_autotools_build_system_constructed (GObject *object)
 {
   IdeAutotoolsBuildSystem *self = (IdeAutotoolsBuildSystem *)object;
+  IdeConfigurationManager *config_manager;
   IdeBufferManager *buffer_manager;
   IdeContext *context;
 
-
   G_OBJECT_CLASS (ide_autotools_build_system_parent_class)->constructed (object);
 
   context = ide_object_get_context (IDE_OBJECT (self));
   buffer_manager = ide_context_get_buffer_manager (context);
+  config_manager = ide_context_get_configuration_manager (context);
+
+  /*
+   * Track change of active configuration so that we invalidate our cached build
+   * targets (which might be out of date).
+   */
+  g_signal_connect_object (config_manager,
+                           "notify::current",
+                           G_CALLBACK (ide_autotools_build_system__config_changed_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   /*
    * FIXME:
@@ -559,6 +568,113 @@ ide_autotools_build_system_constructed (GObject *object)
                            G_CONNECT_SWAPPED);
 }
 
+static void
+ide_autotools_build_system_get_build_targets_cb2 (GObject      *object,
+                                                  GAsyncResult *result,
+                                                  gpointer      user_data)
+{
+  IdeMakecache *makecache = (IdeMakecache *)object;
+  g_autoptr(GTask) task = user_data;
+  GPtrArray *ret;
+  GError *error = NULL;
+
+  g_assert (IDE_IS_MAKECACHE (makecache));
+  g_assert (G_IS_TASK (task));
+
+  ret = ide_makecache_get_build_targets_finish (makecache, result, &error);
+
+  if (ret == NULL)
+    {
+      g_task_return_error (task, error);
+      return;
+    }
+
+  g_task_return_pointer (task, ret, (GDestroyNotify)g_ptr_array_unref);
+}
+
+static void
+ide_autotools_build_system_get_build_targets_cb (GObject      *object,
+                                                 GAsyncResult *result,
+                                                 gpointer      user_data)
+{
+  IdeAutotoolsBuildSystem *self = (IdeAutotoolsBuildSystem *)object;
+  IdeContext *context;
+  IdeVcs *vcs;
+  g_autoptr(IdeConfiguration) configuration = NULL;
+  g_autoptr(IdeBuilder) builder = NULL;
+  g_autoptr(GFile) build_dir = NULL;
+  g_autoptr(IdeMakecache) makecache = NULL;
+  g_autoptr(GTask) task = user_data;
+  GError *error = NULL;
+
+  g_assert (IDE_IS_AUTOTOOLS_BUILD_SYSTEM (self));
+  g_assert (G_IS_TASK (task));
+
+  makecache = ide_autotools_build_system_get_makecache_finish (self, result, &error);
+
+  if (makecache == NULL)
+    {
+      g_task_return_error (task, error);
+      return;
+    }
+
+  context = ide_object_get_context (IDE_OBJECT (self));
+  configuration = ide_configuration_new (context, "autotools-bootstrap", "local", "host");
+  builder = ide_autotools_build_system_get_builder (IDE_BUILD_SYSTEM (self), configuration, &error);
+  if (builder)
+    {
+      build_dir = ide_autotools_builder_get_build_directory (IDE_AUTOTOOLS_BUILDER (builder));
+    }
+  else
+    {
+      vcs = ide_context_get_vcs (context);
+      build_dir = ide_vcs_get_working_directory (vcs);
+    }
+
+  ide_makecache_get_build_targets_async (makecache,
+                                         build_dir,
+                                         g_task_get_cancellable (task),
+                                         ide_autotools_build_system_get_build_targets_cb2,
+                                         g_object_ref (task));
+}
+
+static void
+ide_autotools_build_system_get_build_targets_async (IdeBuildSystem      *build_system,
+                                                    GCancellable        *cancellable,
+                                                    GAsyncReadyCallback  callback,
+                                                    gpointer             user_data)
+{
+  IdeAutotoolsBuildSystem *self = (IdeAutotoolsBuildSystem *)build_system;
+  g_autoptr(GTask) task = NULL;
+
+  g_assert (IDE_IS_AUTOTOOLS_BUILD_SYSTEM (self));
+  g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, ide_autotools_build_system_get_build_targets_async);
+
+  ide_autotools_build_system_get_makecache_async (self,
+                                                  cancellable,
+                                                  ide_autotools_build_system_get_build_targets_cb,
+                                                  g_object_ref (task));
+}
+
+static GPtrArray *
+ide_autotools_build_system_get_build_targets_finish (IdeBuildSystem  *build_system,
+                                                     GAsyncResult    *result,
+                                                     GError         **error)
+{
+  IdeAutotoolsBuildSystem *self = (IdeAutotoolsBuildSystem *)build_system;
+  GTask *task = (GTask *)result;
+
+  g_assert (IDE_IS_AUTOTOOLS_BUILD_SYSTEM (self));
+  g_assert (G_IS_TASK (task));
+  g_assert (g_task_is_valid (task, self));
+  g_assert (g_task_get_source_tag (task) == ide_autotools_build_system_get_build_targets_async);
+
+  return g_task_propagate_pointer (task, error);
+}
+
 static gint
 ide_autotools_build_system_get_priority (IdeBuildSystem *system)
 {
@@ -627,6 +743,8 @@ build_system_iface_init (IdeBuildSystemInterface *iface)
   iface->get_builder = ide_autotools_build_system_get_builder;
   iface->get_build_flags_async = ide_autotools_build_system_get_build_flags_async;
   iface->get_build_flags_finish = ide_autotools_build_system_get_build_flags_finish;
+  iface->get_build_targets_async = ide_autotools_build_system_get_build_targets_async;
+  iface->get_build_targets_finish = ide_autotools_build_system_get_build_targets_finish;
 }
 
 static void
@@ -678,6 +796,8 @@ ide_autotools_build_system_init (IdeAutotoolsBuildSystem *self)
                                          populate_cache_cb,
                                          self,
                                          NULL);
+
+  egg_task_cache_set_name (self->task_cache, "makecache");
 }
 
 static void
