@@ -28,6 +28,9 @@
 
 #include "ide-autotools-build-task.h"
 
+#define FLAG_SET(_f,_n) (((_f) & (_n)) != 0)
+#define FLAG_UNSET(_f,_n) (((_f) & (_n)) == 0)
+
 struct _IdeAutotoolsBuildTask
 {
   IdeBuildResult    parent;
@@ -214,6 +217,7 @@ ide_autotools_build_task_finalize (GObject *object)
 
   g_clear_object (&self->directory);
   g_clear_object (&self->configuration);
+  g_clear_pointer (&self->extra_targets, g_ptr_array_unref);
 
   G_OBJECT_CLASS (ide_autotools_build_task_parent_class)->finalize (object);
 }
@@ -364,7 +368,9 @@ gen_configure_argv (IdeAutotoolsBuildTask *self,
   system_type = ide_device_get_system_type (device);
   g_ptr_array_add (ar, g_strdup_printf ("--host=%s", system_type));
 
-  if (NULL != (opts = ide_configuration_get_config_opts (self->configuration)))
+  opts = ide_configuration_get_config_opts (self->configuration);
+
+  if (!ide_str_empty0 (opts))
     {
       GError *error = NULL;
       gint argc;
@@ -389,8 +395,9 @@ gen_configure_argv (IdeAutotoolsBuildTask *self,
 }
 
 static WorkerState *
-worker_state_new (IdeAutotoolsBuildTask *self,
-                  IdeBuilderBuildFlags   flags)
+worker_state_new (IdeAutotoolsBuildTask  *self,
+                  IdeBuilderBuildFlags    flags,
+                  GError                **error)
 {
   g_autofree gchar *name = NULL;
   IdeContext *context;
@@ -411,6 +418,28 @@ worker_state_new (IdeAutotoolsBuildTask *self,
   device = ide_configuration_get_device (self->configuration);
   runtime = ide_configuration_get_runtime (self->configuration);
 
+  if (device == NULL)
+    {
+      g_set_error (error,
+                   IDE_DEVICE_ERROR,
+                   IDE_DEVICE_ERROR_NO_SUCH_DEVICE,
+                   "%s “%s”",
+                   _("Failed to locate device"),
+                   ide_configuration_get_device_id (self->configuration));
+      return NULL;
+    }
+
+  if (runtime == NULL)
+    {
+      g_set_error (error,
+                   IDE_RUNTIME_ERROR,
+                   IDE_RUNTIME_ERROR_NO_SUCH_RUNTIME,
+                   "%s “%s”",
+                   _("Failed to locate runtime"),
+                   ide_configuration_get_runtime_id (self->configuration));
+      return NULL;
+    }
+
   name = g_file_get_basename (project_file);
 
   if (g_str_has_prefix (name, "configure."))
@@ -420,14 +449,14 @@ worker_state_new (IdeAutotoolsBuildTask *self,
 
   state = g_slice_new0 (WorkerState);
   state->sequence = ide_configuration_get_sequence (self->configuration);
-  state->require_autogen = self->require_autogen || !!(flags & IDE_BUILDER_BUILD_FLAGS_FORCE_BOOTSTRAP);
-  state->require_configure = self->require_configure || (state->require_autogen && !(flags & IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE));
+  state->require_autogen = self->require_autogen || FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_FORCE_BOOTSTRAP);
+  state->require_configure = self->require_configure || (state->require_autogen && FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE));
   state->directory_path = g_file_get_path (self->directory);
   state->project_path = g_file_get_path (project_dir);
   state->system_type = g_strdup (ide_device_get_system_type (device));
   state->runtime = g_object_ref (runtime);
 
-  val32 = atoi (ide_configuration_getenv (self->configuration, "PARALLEL") ?: "-1");
+  val32 = ide_configuration_get_parallelism (self->configuration);
 
   if (val32 == -1)
     state->parallel = g_strdup_printf ("-j%u", g_get_num_processors () + 1);
@@ -438,21 +467,34 @@ worker_state_new (IdeAutotoolsBuildTask *self,
 
   make_targets = g_ptr_array_new ();
 
-  if (0 != (flags & IDE_BUILDER_BUILD_FLAGS_FORCE_CLEAN))
+  if (FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_FORCE_CLEAN))
     {
-      state->require_autogen = TRUE;
-      state->require_configure = TRUE;
+      if (FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
+        {
+          state->require_autogen = TRUE;
+          state->require_configure = TRUE;
+        }
       g_ptr_array_add (make_targets, g_strdup ("clean"));
     }
 
-  if (0 == (flags & IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
+  if (FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
     g_ptr_array_add (make_targets, g_strdup ("all"));
 
+  if (self->extra_targets != NULL)
+    {
+      for (guint i = 0; i < self->extra_targets->len; i++)
+        {
+          const gchar *target = g_ptr_array_index (self->extra_targets, i);
+
+          g_ptr_array_add (make_targets, g_strdup (target));
+        }
+    }
+
   g_ptr_array_add (make_targets, NULL);
 
   state->make_targets = (gchar **)g_ptr_array_free (make_targets, FALSE);
 
-  if (0 != (flags & IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE))
+  if (FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE))
     {
       state->require_autogen = TRUE;
       state->require_configure = TRUE;
@@ -535,25 +577,34 @@ ide_autotools_build_task_execute_async (IdeAutotoolsBuildTask *self,
 {
   g_autoptr(GTask) task = NULL;
   WorkerState *state;
+  GError *error = NULL;
 
   g_return_if_fail (IDE_IS_AUTOTOOLS_BUILD_TASK (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, ide_autotools_build_task_execute_async);
+
   if (self->executed)
     {
-      g_task_report_new_error (self, callback, user_data,
-                               ide_autotools_build_task_execute_async,
+      g_task_return_new_error (task,
                                G_IO_ERROR,
                                G_IO_ERROR_FAILED,
-                               _("Cannot execute build task more than once."));
+                               "%s",
+                               _("Cannot execute build task more than once"));
       return;
     }
 
   self->executed = TRUE;
 
-  state = worker_state_new (self, flags);
+  state = worker_state_new (self, flags, &error);
+
+  if (state == NULL)
+    {
+      g_task_return_error (task, error);
+      return;
+    }
 
-  task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_task_data (task, state, worker_state_free);
 
   /*
@@ -573,6 +624,7 @@ ide_autotools_build_task_execute_finish (IdeAutotoolsBuildTask  *self,
   GTask *task = (GTask *)result;
   WorkerState *state;
   guint sequence;
+  gboolean ret;
 
   g_return_val_if_fail (IDE_IS_AUTOTOOLS_BUILD_TASK (self), FALSE);
   g_return_val_if_fail (G_IS_TASK (task), FALSE);
@@ -583,7 +635,32 @@ ide_autotools_build_task_execute_finish (IdeAutotoolsBuildTask  *self,
   if ((state != NULL) &&  (state->sequence == sequence))
     ide_configuration_set_dirty (self->configuration, FALSE);
 
-  return g_task_propagate_boolean (task, error);
+  ret = g_task_propagate_boolean (task, error);
+
+  /* Mark the task as failed */
+  if (ret == FALSE)
+    ide_build_result_set_failed (IDE_BUILD_RESULT (self), TRUE);
+
+  ide_build_result_set_running (IDE_BUILD_RESULT (self), FALSE);
+
+  return ret;
+}
+
+static gboolean
+log_in_main (gpointer data)
+{
+  struct {
+    IdeBuildResult *result;
+    gchar *message;
+  } *pair = data;
+
+  ide_build_result_log_stdout (pair->result, "%s", pair->message);
+
+  g_free (pair->message);
+  g_object_unref (pair->result);
+  g_slice_free1 (sizeof *pair, pair);
+
+  return G_SOURCE_REMOVE;
 }
 
 static GSubprocess *
@@ -595,9 +672,14 @@ log_and_spawn (IdeAutotoolsBuildTask  *self,
                ...)
 {
   GSubprocess *ret;
+  struct {
+    IdeBuildResult *result;
+    gchar *message;
+  } *pair;
   GString *log;
   gchar *item;
   va_list args;
+  gint popcnt = 0;
 
   g_assert (IDE_IS_AUTOTOOLS_BUILD_TASK (self));
   g_assert (IDE_IS_SUBPROCESS_LAUNCHER (launcher));
@@ -611,12 +693,23 @@ log_and_spawn (IdeAutotoolsBuildTask  *self,
     {
       ide_subprocess_launcher_push_argv (launcher, item);
       g_string_append_printf (log, " '%s'", item);
+      popcnt++;
     }
   va_end (args);
 
-  ide_build_result_log_stdout (IDE_BUILD_RESULT (self), "%s", log->str);
+  pair = g_slice_alloc (sizeof *pair);
+  pair->result = g_object_ref (self);
+  pair->message = g_string_free (log, FALSE);
+  g_timeout_add (0, log_in_main, pair);
+
   ret = ide_subprocess_launcher_spawn_sync (launcher, cancellable, error);
-  g_string_free (log, TRUE);
+
+  /* pop make args */
+  for (; popcnt; popcnt--)
+    g_free (ide_subprocess_launcher_pop_argv (launcher));
+
+  /* pop "make" */
+  g_free (ide_subprocess_launcher_pop_argv (launcher));
 
   return ret;
 }
@@ -904,3 +997,16 @@ apply_environment (IdeAutotoolsBuildTask *self,
   environment = ide_configuration_get_environment (self->configuration);
   ide_subprocess_launcher_overlay_environment (launcher, environment);
 }
+
+void
+ide_autotools_build_task_add_target (IdeAutotoolsBuildTask *self,
+                                     const gchar           *target)
+{
+  g_return_if_fail (IDE_IS_AUTOTOOLS_BUILD_TASK (self));
+  g_return_if_fail (target != NULL);
+
+  if (self->extra_targets == NULL)
+    self->extra_targets = g_ptr_array_new_with_free_func (g_free);
+
+  g_ptr_array_add (self->extra_targets, g_strdup (target));
+}
