@@ -23,10 +23,8 @@
 #include <gtksourceview/gtksource.h>
 #include <ide.h>
 
-#include "ide-gtk.h"
-#include "ide-editor-frame.h"
-#include "ide-editor-frame-private.h"
-#include "ide-editor-view-private.h"
+#include "editor/ide-editor-frame-private.h"
+#include "editor/ide-editor-view-private.h"
 
 #include "gb-vim.h"
 
@@ -60,6 +58,12 @@ typedef struct
   gchar            *options_sup;
 } GbVimCommand;
 
+typedef struct
+{
+  GtkWidget   *active_widget;
+  gchar *file_path;
+} SplitCallbackData;
+
 static gboolean
 int32_parse (gint         *value,
              const gchar  *str,
@@ -469,7 +473,7 @@ gb_vim_command_edit (GtkWidget      *active_widget,
   else
     file = g_file_get_child (workdir, options);
 
-  ide_workbench_open_files_async (workbench, &file, 1, "editor", NULL, NULL, NULL);
+  ide_workbench_open_files_async (workbench, &file, 1, "editor", IDE_WORKBENCH_OPEN_FLAGS_NONE, NULL, NULL, NULL);
 
   g_clear_object (&file);
 
@@ -528,19 +532,82 @@ gb_vim_command_split (GtkWidget      *active_widget,
   return TRUE;
 }
 
+static void
+gb_vim_command_vsplit_cb (GObject      *object,
+                          GAsyncResult *result,
+                          gpointer      user_data)
+{
+  SplitCallbackData *split_callback_data = (SplitCallbackData *)user_data;
+  GtkWidget *active_widget;
+  const gchar *file_path;
+  GVariant *variant;
+
+  active_widget = split_callback_data->active_widget;
+  file_path = split_callback_data->file_path;
+  variant = g_variant_new_string (file_path);
+
+  ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "split-left", variant);
+
+  g_object_unref (split_callback_data->active_widget);
+  g_free (split_callback_data->file_path);
+  g_slice_free (SplitCallbackData, split_callback_data);
+}
+
 static gboolean
 gb_vim_command_vsplit (GtkWidget      *active_widget,
                        const gchar    *command,
                        const gchar    *options,
                        GError        **error)
 {
+  IdeWorkbench *workbench;
+  IdeContext *context;
+  IdeVcs *vcs;
+  GFile *workdir;
+  GFile *file = NULL;
+  gchar *file_path;
+  SplitCallbackData *split_callback_data;
+  GVariant *variant;
+
   g_assert (GTK_IS_WIDGET (active_widget));
 
-  if (IDE_IS_LAYOUT_VIEW (active_widget))
-    ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "split-left", NULL);
-  else
+  if (!IDE_IS_LAYOUT_VIEW (active_widget))
     return gb_vim_set_no_view_error (error);
 
+  if (ide_str_empty0 (options))
+    {
+      variant = g_variant_new_string ("");
+      ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "split-left", variant);
+    }
+  else
+    {
+      if (!(workbench = ide_widget_get_workbench (active_widget)) ||
+          !(context = ide_workbench_get_context (workbench)) ||
+          !(vcs = ide_context_get_vcs (context)) ||
+          !(workdir = ide_vcs_get_working_directory (vcs)))
+        {
+          g_set_error (error,
+                       GB_VIM_ERROR,
+                       GB_VIM_ERROR_NOT_SOURCE_VIEW,
+                       _("Failed to locate working directory"));
+          return FALSE;
+        }
+
+      file_path = g_strdup (options);
+
+      if (!g_path_is_absolute (file_path))
+        file_path = g_build_filename (g_file_get_path (workdir), file_path, NULL);
+
+      file = g_file_new_for_path (file_path);
+
+      split_callback_data = g_slice_new (SplitCallbackData);
+      split_callback_data->active_widget = g_object_ref (active_widget);
+      split_callback_data->file_path = file_path;
+
+      ide_workbench_open_files_async (workbench, &file, 1, "editor", IDE_WORKBENCH_OPEN_FLAGS_BACKGROUND, NULL, gb_vim_command_vsplit_cb, split_callback_data);
+
+      g_clear_object (&file);
+    }
+
   return TRUE;
 }
 
@@ -677,10 +744,20 @@ gb_vim_command_bnext (GtkWidget      *active_widget,
                       const gchar    *options,
                       GError        **error)
 {
+  IdeWorkbench *workbench;
+  IdeContext *context;
+  IdeBufferManager *bufmgr;
+  guint n_buffers;
+
   g_assert (GTK_IS_WIDGET (active_widget));
 
-  // TODO: check for an open buffer ?
-  ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "next-view", NULL);
+  workbench = ide_widget_get_workbench (GTK_WIDGET (active_widget));
+  context = ide_workbench_get_context (workbench);
+  bufmgr = ide_context_get_buffer_manager (context);
+  n_buffers = ide_buffer_manager_get_n_buffers (bufmgr);
+
+  if (n_buffers > 0)
+    ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "next-view", NULL);
 
   return TRUE;
 }
@@ -691,10 +768,20 @@ gb_vim_command_bprevious (GtkWidget      *active_widget,
                           const gchar    *options,
                           GError        **error)
 {
+  IdeWorkbench *workbench;
+  IdeContext *context;
+  IdeBufferManager *bufmgr;
+  guint n_buffers;
+
   g_assert (GTK_IS_WIDGET (active_widget));
 
-  // TODO: check for an open buffer ?
-  ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "previous-view", NULL);
+  workbench = ide_widget_get_workbench (GTK_WIDGET (active_widget));
+  context = ide_workbench_get_context (workbench);
+  bufmgr = ide_context_get_buffer_manager (context);
+  n_buffers = ide_buffer_manager_get_n_buffers (bufmgr);
+
+  if (n_buffers > 0)
+    ide_widget_action (GTK_WIDGET (active_widget), "view-stack", "previous-view", NULL);
 
   return TRUE;
 }
@@ -859,6 +946,7 @@ gb_vim_do_search_and_replace (GtkTextBuffer *buffer,
   GtkTextIter tmp2;
   GtkTextIter match_begin;
   GtkTextIter match_end;
+  gboolean has_wrapped = FALSE;
   GError *error = NULL;
 
   g_assert (search_text);
@@ -885,25 +973,21 @@ gb_vim_do_search_and_replace (GtkTextBuffer *buffer,
   gtk_source_search_settings_set_search_text (search_settings, search_text);
   gtk_source_search_settings_set_case_sensitive (search_settings, TRUE);
 
-  while (gtk_source_search_context_forward (search_context, begin, &match_begin, &match_end))
+  while (gtk_source_search_context_forward2 (search_context,
+                                             begin,
+                                             &match_begin,
+                                             &match_end,
+                                             &has_wrapped) && !has_wrapped)
     {
       if (is_global || gb_vim_match_is_selected (buffer, &match_begin, &match_end))
         {
-          GtkTextMark *mark2;
-
-          mark2 = gtk_text_buffer_create_mark (buffer, NULL, &match_end, FALSE);
-
-          if (!gtk_source_search_context_replace (search_context, &match_begin, &match_end,
-                                                  replace_text, -1, &error))
+          if (!gtk_source_search_context_replace2 (search_context, &match_begin, &match_end,
+                                                   replace_text, -1, &error))
             {
               g_warning ("%s", error->message);
               g_clear_error (&error);
-              gtk_text_buffer_delete_mark (buffer, mark2);
               break;
             }
-
-          gtk_text_buffer_get_iter_at_mark (buffer, &match_end, mark2);
-          gtk_text_buffer_delete_mark (buffer, mark2);
         }
 
       *begin = match_end;
@@ -932,6 +1016,7 @@ gb_vim_command_search (GtkWidget      *active_widget,
   gchar *search_text = NULL;
   gchar *replace_text = NULL;
   gunichar separator;
+  gboolean confirm_replace = FALSE;
 
   g_assert (GTK_IS_WIDGET (active_widget));
   g_assert (g_str_has_prefix (command, "%s") || g_str_has_prefix (command, "s"));
@@ -1001,6 +1086,10 @@ gb_vim_command_search (GtkWidget      *active_widget,
         {
           switch (*command)
             {
+            case 'c':
+              confirm_replace = TRUE;
+              break;
+
             case 'g':
               break;
 
@@ -1014,6 +1103,23 @@ gb_vim_command_search (GtkWidget      *active_widget,
   search_text = g_strndup (search_begin, search_end - search_begin);
   replace_text = g_strndup (replace_begin, replace_end - replace_begin);
 
+  if (confirm_replace)
+    {
+      GVariant *variant;
+      GVariantBuilder builder;
+
+      g_variant_builder_init (&builder, G_VARIANT_TYPE_STRING_ARRAY);
+      g_variant_builder_add (&builder, "s", search_text);
+      g_variant_builder_add (&builder, "s", replace_text);
+      variant = g_variant_builder_end (&builder);
+
+      ide_widget_action (GTK_WIDGET (IDE_EDITOR_VIEW (active_widget)->frame1),
+                         "frame",
+                         "replace-confirm",
+                         variant);
+      return TRUE;
+    }
+
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (source_view));
 
   if (gtk_text_buffer_get_has_selection (buffer))
@@ -1046,14 +1152,15 @@ static const GbVimCommand vim_commands[] = {
   { "bnext",       gb_vim_command_bnext , NULL},
   { "bprevious",   gb_vim_command_bprevious, NULL },
   { "buffers",     gb_vim_command_buffers, NULL },
-  { "ls",          gb_vim_command_buffers, NULL },
   { "cnext",       gb_vim_command_cnext, NULL },
   { "colorscheme", gb_vim_command_colorscheme, NULL },
   { "cprevious",   gb_vim_command_cprevious, NULL },
   { "edit",        gb_vim_command_edit, NULL },
   { "help",        gb_vim_command_help, NULL },
-  { "nohl",        gb_vim_command_nohl, NULL },
+  { "ls",          gb_vim_command_buffers, NULL },
   { "make",        gb_vim_command_make, NULL },
+  { "nohl",        gb_vim_command_nohl, NULL },
+  { "open",        gb_vim_command_edit, NULL },
   { "quit",        gb_vim_command_quit, NULL },
   { "set",         gb_vim_command_set, NULL },
   { "sort",        gb_vim_command_sort, NULL },
@@ -1291,18 +1398,10 @@ gb_vim_complete_edit_files (GtkWidget *active_widget,
   if (parent != NULL)
     {
       g_autoptr(GFileEnumerator) fe = NULL;
-      g_autofree gchar *relpath = NULL;
       GFileInfo *descendent;
       const gchar *slash;
-
-      relpath = g_file_get_relative_path (workdir, parent);
-
-      if (relpath && g_str_has_prefix (relpath, "./"))
-        {
-          gchar *tmp = relpath;
-          relpath = g_strdup (relpath + 2);
-          g_free (tmp);
-        }
+      const gchar *partial_name;
+      g_autofree gchar *prefix_dir = NULL;
 
 #ifdef IDE_ENABLE_TRACE
       {
@@ -1312,7 +1411,14 @@ gb_vim_complete_edit_files (GtkWidget *active_widget,
 #endif
 
       if ((slash = strrchr (prefix, G_DIR_SEPARATOR)))
-        prefix = slash + 1;
+        {
+          partial_name = slash + 1;
+          prefix_dir = g_strndup (prefix, slash - prefix + 1);
+        }
+      else
+        {
+          partial_name = prefix;
+        }
 
       fe = g_file_enumerate_children (parent,
                                       G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
@@ -1325,23 +1431,31 @@ gb_vim_complete_edit_files (GtkWidget *active_widget,
       while ((descendent = g_file_enumerator_next_file (fe, NULL, NULL)))
         {
           const gchar *name;
-
           name = g_file_info_get_display_name (descendent);
 
           IDE_TRACE_MSG ("name=%s prefix=%s", name, prefix);
 
-          if (name && g_str_has_prefix (name, prefix))
+          if (name && g_str_has_prefix (name, partial_name))
             {
-              gchar *path;
+              gchar *completed_command;
+              const gchar *descendent_name;
+              g_autofree gchar *full_path = NULL;
+              g_autofree gchar *parent_path = NULL;
 
-              if (relpath)
-                path = g_strdup_printf ("%s %s/%s", command, relpath, name);
+              parent_path = g_file_get_path (parent);
+              descendent_name = g_file_info_get_name (descendent);
+              full_path = g_build_filename (parent_path, descendent_name, NULL);
+
+              if (prefix[0] == G_DIR_SEPARATOR)
+                completed_command = g_strdup_printf ("%s %s", command, full_path);
+              else if (strchr (prefix, G_DIR_SEPARATOR) == NULL)
+                completed_command = g_strdup_printf ("%s %s", command, descendent_name);
               else
-                path = g_strdup_printf ("%s %s", command, name);
+                completed_command = g_strdup_printf ("%s %s%s", command, prefix_dir, descendent_name);
 
-              IDE_TRACE_MSG ("edit completion: %s", path);
+              IDE_TRACE_MSG ("edit completion: %s", completed_command);
 
-              g_ptr_array_add (ar, path);
+              g_ptr_array_add (ar, completed_command);
             }
           g_object_unref (descendent);
         }
@@ -1433,6 +1547,10 @@ gb_vim_complete (GtkWidget   *active_widget,
 
       if (g_str_has_prefix (line, "e ") ||
           g_str_has_prefix (line, "edit ") ||
+          g_str_has_prefix (line, "o ") ||
+          g_str_has_prefix (line, "open ") ||
+          g_str_has_prefix (line, "vsp ") ||
+          g_str_has_prefix (line, "vsplit ") ||
           g_str_has_prefix (line, "tabe "))
           gb_vim_complete_edit (active_widget, line, ar);
       else
