@@ -16,10 +16,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "gbp-create-project-widget"
+
+#include <egg-file-chooser-entry.h>
+#include <egg-radio-box.h>
+#include <glib/gi18n.h>
+#include <ide.h>
 #include <libpeas/peas.h>
 #include <stdlib.h>
 
-#include "ide-macros.h"
 #include "gbp-create-project-template-icon.h"
 #include "gbp-create-project-widget.h"
 
@@ -28,10 +33,11 @@ struct _GbpCreateProjectWidget
   GtkBin                parent;
 
   GtkEntry             *project_name_entry;
-  GtkEntry             *project_location_entry;
-  GtkFileChooserButton *project_location_button;
-  GtkComboBoxText      *project_language_chooser;
+  EggFileChooserEntry  *project_location_entry;
+  EggRadioBox          *project_language_chooser;
   GtkFlowBox           *project_template_chooser;
+  GtkSwitch            *versioning_switch;
+  EggRadioBox          *license_chooser;
 };
 
 enum {
@@ -84,7 +90,7 @@ gbp_create_project_widget_add_languages (GbpCreateProjectWidget *self,
   keys = (const gchar **)g_hash_table_get_keys_as_array (languages, &len);
   qsort (keys, len, sizeof (gchar *), sort_by_name);
   for (i = 0; keys [i]; i++)
-    gtk_combo_box_text_append (self->project_language_chooser, NULL, keys [i]);
+    egg_radio_box_add_item (self->project_language_chooser, keys [i], keys [i]);
   g_free (keys);
 }
 
@@ -122,7 +128,7 @@ gbp_create_project_widget_name_changed (GbpCreateProjectWidget *self,
                     "secondary-icon-name", "dialog-warning-symbolic",
                     NULL);
 
-      gtk_entry_set_text (self->project_location_entry, "");
+      project_dir = g_strdup ("");
     }
   else
     {
@@ -131,7 +137,6 @@ gbp_create_project_widget_name_changed (GbpCreateProjectWidget *self,
                     NULL);
 
       project_dir = g_ascii_strdown (g_strdelimit (project_name, " ", '-'), -1);
-      gtk_entry_set_text (self->project_location_entry, project_dir);
     }
 
   g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_IS_READY]);
@@ -144,14 +149,14 @@ gbp_create_project_widget_flow_box_filter (GtkFlowBoxChild *template_container,
   GbpCreateProjectWidget *self = object;
   GbpCreateProjectTemplateIcon *template_icon;
   IdeProjectTemplate *template;
-  g_autofree gchar *language = NULL;
+  const gchar *language = NULL;
   g_auto(GStrv) template_languages = NULL;
   gint i;
 
   g_assert (GTK_IS_FLOW_BOX_CHILD (template_container));
   g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
 
-  language = gtk_combo_box_text_get_active_text (self->project_language_chooser);
+  language = egg_radio_box_get_active_id (self->project_language_chooser);
 
   if (ide_str_empty0 (language))
     return TRUE;
@@ -176,9 +181,10 @@ gbp_create_project_widget_flow_box_filter (GtkFlowBoxChild *template_container,
 
 static void
 gbp_create_project_widget_language_changed (GbpCreateProjectWidget *self,
-                                            GtkComboBox            *language_chooser)
+                                            EggRadioBox            *language_chooser)
 {
   g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+  g_assert (EGG_IS_RADIO_BOX (language_chooser));
 
   gtk_flow_box_invalidate_filter (self->project_template_chooser);
 
@@ -231,16 +237,42 @@ template_providers_foreach_cb (PeasExtensionSet *set,
                                gpointer          user_data)
 {
   GbpCreateProjectWidget *self = user_data;
-  IdeTemplateProvider *provider = IDE_TEMPLATE_PROVIDER (exten);
-  GList *templates = ide_template_provider_get_project_templates (provider);
+  IdeTemplateProvider *provider = (IdeTemplateProvider *)exten;
+  GList *templates;
 
+  g_assert (PEAS_IS_EXTENSION_SET (set));
   g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+  g_assert (IDE_IS_TEMPLATE_PROVIDER (provider));
+
+  templates = ide_template_provider_get_project_templates (provider);
 
   gbp_create_project_widget_add_template_buttons (self, templates);
   gbp_create_project_widget_add_languages (self, templates);
 
-  g_list_foreach (templates, (GFunc)g_object_unref, NULL);
-  g_list_free (templates);
+  g_list_free_full (templates, g_object_unref);
+}
+
+static GFile *
+gbp_create_project_widget_get_directory (GbpCreateProjectWidget *self)
+{
+  g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+
+  return egg_file_chooser_entry_get_file (self->project_location_entry);
+}
+
+static void
+gbp_create_project_widget_set_directory (GbpCreateProjectWidget *self,
+                                         const gchar            *path)
+{
+  g_autofree gchar *resolved = NULL;
+  g_autoptr(GFile) file = NULL;
+
+  g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+
+  resolved = ide_path_expand (path);
+  file = g_file_new_for_path (resolved);
+
+  egg_file_chooser_entry_set_file (self->project_location_entry, file);
 }
 
 static void
@@ -251,18 +283,15 @@ gbp_create_project_widget_constructed (GObject *object)
   PeasExtensionSet *extensions;
 
   engine = peas_engine_get_default ();
-  extensions = peas_extension_set_new (engine,
-                                       IDE_TYPE_TEMPLATE_PROVIDER,
-                                       NULL);
-  peas_extension_set_foreach (extensions,
-                              template_providers_foreach_cb,
-                              self);
 
+  /* Load templates */
+  extensions = peas_extension_set_new (engine, IDE_TYPE_TEMPLATE_PROVIDER, NULL);
+  peas_extension_set_foreach (extensions, template_providers_foreach_cb, self);
   g_clear_object (&extensions);
 
   G_OBJECT_CLASS (gbp_create_project_widget_parent_class)->constructed (object);
 
-  gtk_combo_box_set_active (GTK_COMBO_BOX (self->project_language_chooser), 0);
+  egg_radio_box_set_active_id (self->project_language_chooser, "C");
 }
 
 static void
@@ -276,7 +305,7 @@ gbp_create_project_widget_is_ready (GbpCreateProjectWidget *self)
 {
   const gchar *text;
   g_autofree gchar *project_name = NULL;
-  g_autofree gchar *language = NULL;
+  const gchar *language = NULL;
   GList *selected_template = NULL;
 
   g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
@@ -287,7 +316,7 @@ gbp_create_project_widget_is_ready (GbpCreateProjectWidget *self)
   if (ide_str_empty0 (project_name) || !validate_name (project_name))
     return FALSE;
 
-  language = gtk_combo_box_text_get_active_text (self->project_language_chooser);
+  language = egg_radio_box_get_active_id (self->project_language_chooser);
 
   if (ide_str_empty0 (language))
     return FALSE;
@@ -344,10 +373,11 @@ gbp_create_project_widget_class_init (GbpCreateProjectWidgetClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/builder/plugins/create-project-plugin/gbp-create-project-widget.ui");
   gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, project_name_entry);
-  gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, project_location_button);
   gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, project_location_entry);
   gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, project_language_chooser);
   gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, project_template_chooser);
+  gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, versioning_switch);
+  gtk_widget_class_bind_template_child (widget_class, GbpCreateProjectWidget, license_chooser);
 }
 
 static void
@@ -355,23 +385,13 @@ gbp_create_project_widget_init (GbpCreateProjectWidget *self)
 {
   g_autoptr(GSettings) settings = NULL;
   g_autofree gchar *path = NULL;
-  g_autofree char *projects_dir = NULL;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
   settings = g_settings_new ("org.gnome.builder");
+
   path = g_settings_get_string (settings, "projects-directory");
-
-  if (!ide_str_empty0 (path))
-    {
-      if (!g_path_is_absolute (path))
-        projects_dir = g_build_filename (g_get_home_dir (), path, NULL);
-      else
-        projects_dir = g_steal_pointer (&path);
-
-      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (self->project_location_button),
-                                           projects_dir);
-    }
+  gbp_create_project_widget_set_directory (self, path);
 
   gtk_flow_box_set_filter_func (self->project_template_chooser,
                                 gbp_create_project_widget_flow_box_filter,
@@ -384,7 +404,6 @@ gbp_create_project_widget_init (GbpCreateProjectWidget *self)
                            self,
                            G_CONNECT_SWAPPED);
 
-
   g_signal_connect_object (self->project_language_chooser,
                            "changed",
                            G_CALLBACK (gbp_create_project_widget_language_changed),
@@ -398,41 +417,110 @@ gbp_create_project_widget_init (GbpCreateProjectWidget *self)
                            G_CONNECT_SWAPPED);
 }
 
+static void
+init_vcs_cb (GObject      *object,
+             GAsyncResult *result,
+             gpointer      user_data)
+{
+  g_autoptr(GTask) task = user_data;
+  IdeVcsInitializer *vcs = (IdeVcsInitializer *)object;
+  GbpCreateProjectWidget *self;
+  IdeWorkbench *workbench;
+  GFile *project_file;
+  GError *error = NULL;
+
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (G_IS_TASK (task));
+
+  if (!ide_vcs_initializer_initialize_finish (vcs, result, &error))
+    {
+      g_task_return_error (task, error);
+      return;
+    }
+
+  self = g_task_get_source_object (task);
+  g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+
+  project_file = g_task_get_task_data (task);
+  g_assert (G_IS_FILE (project_file));
+
+  workbench = ide_widget_get_workbench (GTK_WIDGET (self));
+  ide_workbench_open_project_async (workbench, project_file, NULL, NULL, NULL);
+
+  g_task_return_boolean (task, TRUE);
+}
+
 static void
 extract_cb (GObject      *object,
             GAsyncResult *result,
             gpointer      user_data)
 {
-  GbpCreateProjectWidget *self;
-  IdeWorkbench *workbench;
   IdeProjectTemplate *template = (IdeProjectTemplate *)object;
   g_autoptr(GTask) task = user_data;
+  g_autoptr(IdeVcsInitializer) vcs = NULL;
+  GbpCreateProjectWidget *self;
+  IdeWorkbench *workbench;
+  PeasEngine *engine;
+  PeasPluginInfo *plugin_info;
+  GFile *project_file;
   GError *error = NULL;
-  const gchar *path;
-  g_autoptr(GFile) project_file = NULL;
+
+  /* To keep the UI simple, we only support git from
+   * the creation today. However, at the time of writing
+   * that is our only supported VCS anyway. If you'd like to
+   * add support for an additional VCS, we need to redesign
+   * this part of the UI.
+   */
+  const gchar *vcs_id = "git-plugin";
 
   g_assert (IDE_IS_PROJECT_TEMPLATE (template));
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (G_IS_TASK (task));
 
-
   if (!ide_project_template_expand_finish (template, result, &error))
     {
-      g_object_unref (template);
       g_task_return_error (task, error);
       return;
     }
-  else
+
+  self = g_task_get_source_object (task);
+  g_assert (GBP_IS_CREATE_PROJECT_WIDGET (self));
+
+  project_file = g_task_get_task_data (task);
+  g_assert (G_IS_FILE (project_file));
+
+  if (!gtk_switch_get_active (self->versioning_switch))
     {
-      self = g_task_get_source_object (task);
-      path = g_task_get_task_data (task);
-      project_file = g_file_new_for_path (path);
       workbench = ide_widget_get_workbench (GTK_WIDGET (self));
       ide_workbench_open_project_async (workbench, project_file, NULL, NULL, NULL);
+      g_task_return_boolean (task, TRUE);
+      return;
     }
 
-  g_object_unref (template);
-  g_task_return_boolean (task, TRUE);
+  engine = peas_engine_get_default ();
+  plugin_info = peas_engine_get_plugin_info (engine, vcs_id);
+  if (plugin_info == NULL)
+    goto failure;
+
+  vcs = (IdeVcsInitializer *)peas_engine_create_extension (engine, plugin_info,
+                                                           IDE_TYPE_VCS_INITIALIZER,
+                                                           NULL);
+  if (vcs == NULL)
+    goto failure;
+
+  ide_vcs_initializer_initialize_async (vcs,
+                                        project_file,
+                                        g_task_get_cancellable (task),
+                                        init_vcs_cb,
+                                        g_object_ref (task));
+
+  return;
+
+failure:
+  g_task_return_new_error (task,
+                           G_IO_ERROR,
+                           G_IO_ERROR_FAILED,
+                           _("A failure occurred while initializing version control"));
 }
 
 void
@@ -443,16 +531,23 @@ gbp_create_project_widget_create_async (GbpCreateProjectWidget *self,
 {
   g_autoptr(GTask) task = NULL;
   g_autoptr(GHashTable) params = NULL;
-  IdeProjectTemplate *template;
-  g_autofree gchar *name;
-  g_autofree gchar *location = NULL;
+  g_autoptr(IdeProjectTemplate) template = NULL;
+  g_autoptr(IdeVcsConfig) vcs_conf = NULL;
+  GValue str = G_VALUE_INIT;
+  g_autofree gchar *name = NULL;
   g_autofree gchar *path = NULL;
-  g_autofree gchar *language = NULL;
+  g_autoptr(GFile) location = NULL;
+  g_autoptr(GFile) child = NULL;
+  const gchar *language = NULL;
+  const gchar *license_id = NULL;
   GtkFlowBoxChild *template_container;
   GbpCreateProjectTemplateIcon *template_icon;
+  PeasEngine *engine;
+  PeasPluginInfo *plugin_info;
   const gchar *text;
-  const gchar *child_name;
-  GList *selected_box_child = NULL;
+  const gchar *vcs_id = "git-plugin";
+  const gchar *author_name;
+  GList *selected_box_child;
 
   g_return_if_fail (GBP_CREATE_PROJECT_WIDGET (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
@@ -476,32 +571,80 @@ gbp_create_project_widget_create_async (GbpCreateProjectWidget *self,
                        g_strdup ("name"),
                        g_variant_ref_sink (g_variant_new_string (g_strdelimit (name, " ", '-'))));
 
-  child_name = gtk_entry_get_text (self->project_location_entry);
-  location = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self->project_location_button));
-
-  if (!ide_str_empty0 (child_name))
-    path = g_build_filename (location, child_name, NULL);
-  else
-    path = g_steal_pointer (&location);
+  location = gbp_create_project_widget_get_directory (self);
+  child = g_file_get_child (location, name);
+  path = g_file_get_path (child);
 
   g_hash_table_insert (params,
                        g_strdup ("path"),
                        g_variant_ref_sink (g_variant_new_string (path)));
 
-  language = gtk_combo_box_text_get_active_text (self->project_language_chooser);
+  language = egg_radio_box_get_active_id (self->project_language_chooser);
   g_hash_table_insert (params,
                        g_strdup ("language"),
                        g_variant_ref_sink (g_variant_new_string (language)));
 
-  task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_task_data (task, g_strdup (path), g_free);
+  license_id = egg_radio_box_get_active_id (EGG_RADIO_BOX (self->license_chooser));
 
-  ide_project_template_expand_async (g_object_ref (template),
+  if (!g_str_equal (license_id, "none"))
+    {
+      g_autofree gchar *license_full_path = NULL;
+      g_autofree gchar *license_short_path = NULL;
+
+      license_full_path = g_strjoin (NULL, "resource://", "/org/gnome/builder/plugins/create-project-plugin/license/full/", license_id, NULL);
+      license_short_path = g_strjoin (NULL, "resource://", "/org/gnome/builder/plugins/create-project-plugin/license/short/", license_id, NULL);
+
+      g_hash_table_insert (params,
+                           g_strdup ("license_full"),
+                           g_variant_ref_sink (g_variant_new_string (license_full_path)));
+
+      g_hash_table_insert (params,
+                           g_strdup ("license_short"),
+                           g_variant_ref_sink (g_variant_new_string (license_short_path)));
+    }
+
+  if (gtk_switch_get_active (self->versioning_switch))
+    {
+      g_hash_table_insert (params,
+                           g_strdup ("versioning"),
+                           g_variant_ref_sink (g_variant_new_string ("git")));
+
+      engine = peas_engine_get_default ();
+      plugin_info = peas_engine_get_plugin_info (engine, vcs_id);
+
+      if (plugin_info != NULL)
+        {
+          vcs_conf = (IdeVcsConfig *)peas_engine_create_extension (engine, plugin_info,
+                                                                   IDE_TYPE_VCS_CONFIG,
+                                                                   NULL);
+
+          if (vcs_conf != NULL)
+            {
+              g_value_init (&str, G_TYPE_STRING);
+              ide_vcs_config_get_config (vcs_conf, IDE_VCS_CONFIG_FULL_NAME, &str);
+            }
+        }
+    }
+
+  if (G_VALUE_HOLDS_STRING (&str) && !ide_str_empty0 (g_value_get_string (&str)))
+    author_name = g_value_get_string (&str);
+  else
+    author_name = g_get_real_name ();
+
+  g_hash_table_insert (params,
+                       g_strdup ("author"),
+                       g_variant_ref_sink (g_variant_new_string (author_name)));
+
+  g_value_unset (&str);
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_task_data (task, g_file_new_for_path (path), g_object_unref);
+
+  ide_project_template_expand_async (template,
                                      params,
                                      NULL,
                                      extract_cb,
                                      g_object_ref (task));
-  g_object_unref (template);
 }
 
 gboolean
